@@ -2,7 +2,7 @@
  *
  *  Connection Manager
  *
- *  Copyright (C) 2007-2008  Intel Corporation. All rights reserved.
+ *  Copyright (C) 2007-2010  Intel Corporation. All rights reserved.
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License version 2 as
@@ -26,14 +26,18 @@
 extern "C" {
 #endif
 
-#include <connman/element.h>
-
 /**
  * SECTION:security
  * @title: Security premitives
  * @short_description: Functions for registering security modules
  */
 
+enum connman_security_privilege {
+	CONNMAN_SECURITY_PRIVILEGE_PUBLIC  = 0,
+	CONNMAN_SECURITY_PRIVILEGE_MODIFY  = 1,
+	CONNMAN_SECURITY_PRIVILEGE_SECRET  = 2,
+};
+
 #define CONNMAN_SECURITY_PRIORITY_LOW      -100
 #define CONNMAN_SECURITY_PRIORITY_DEFAULT     0
 #define CONNMAN_SECURITY_PRIORITY_HIGH      100
@@ -41,11 +45,12 @@ extern "C" {
 struct connman_security {
 	const char *name;
 	int priority;
-	int (*authorize_sender) (const char *sender);
+	int (*authorize_sender) (const char *sender,
+				enum connman_security_privilege privilege);
 };
 
-extern int connman_security_register(struct connman_security *security);
-extern void connman_security_unregister(struct connman_security *security);
+int connman_security_register(struct connman_security *security);
+void connman_security_unregister(struct connman_security *security);
 
 #ifdef __cplusplus
 }
