@@ -38,6 +38,7 @@ struct gateway_data {
 	/* VPN extra data */
 	gboolean vpn;
 	char *vpn_ip;
+	int vpn_phy_index;
 };
 
 static GSList *gateway_list = NULL;
@@ -65,16 +66,19 @@ static struct gateway_data *find_gateway(int index, const char *gateway)
 
 static int del_routes(struct gateway_data *data)
 {
-	const char *address;
-
-	connman_inet_del_host_route(data->index, data->gateway);
-
-	if (data->vpn)
-		address = data->vpn_ip;
-	else
-		address = data->gateway;
-
-	return connman_inet_clear_gateway_address(data->index, address);
+	if (data->vpn) {
+		if (data->vpn_phy_index >= 0)
+			connman_inet_del_host_route(data->vpn_phy_index,
+							data->gateway);
+		return connman_inet_clear_gateway_address(data->index,
+							data->vpn_ip);
+	} else if (g_strcmp0(data->gateway, "0.0.0.0") == 0) {
+		return connman_inet_clear_gateway_interface(data->index);
+	} else {
+		connman_inet_del_host_route(data->index, data->gateway);
+		return connman_inet_clear_gateway_address(data->index,
+							data->gateway);
+	}
 }
 
 static void find_element(struct connman_element *element, gpointer user_data)
@@ -107,6 +111,7 @@ static struct gateway_data *add_gateway(int index, const char *gateway)
 	data->element = NULL;
 	data->vpn_ip = NULL;
 	data->vpn = FALSE;
+	data->vpn_phy_index = -1;
 
 	__connman_element_foreach(NULL, CONNMAN_ELEMENT_TYPE_CONNECTION,
 							find_element, data);
@@ -136,23 +141,17 @@ static void set_default_gateway(struct gateway_data *data)
 {
 	struct connman_element *element = data->element;
 	struct connman_service *service = NULL;
-	short int ifflags;
 
 	DBG("gateway %s", data->gateway);
 
 	if (data->vpn == TRUE) {
 		connman_inet_set_gateway_address(data->index, data->vpn_ip);
+		data->active = TRUE;
 		/* vpn gateway going away no changes in services */
 		return;
 	}
 
-	ifflags = connman_inet_ifflags(element->index);
-	if (ifflags < 0) {
-		connman_error("Fail to get network interface flags");
-		return;
-	}
-
-	if (ifflags & IFF_POINTOPOINT) {
+	if (g_strcmp0(data->gateway, "0.0.0.0") == 0) {
 		if (connman_inet_set_gateway_interface(element->index) < 0)
 			return;
 		goto done;
@@ -186,18 +185,24 @@ static struct gateway_data *find_default_gateway(void)
 	return found;
 }
 
-static void remove_gateway(struct gateway_data *data)
+static int remove_gateway(struct gateway_data *data)
 {
+	int err;
+
 	DBG("gateway %s", data->gateway);
 
 	gateway_list = g_slist_remove(gateway_list, data);
 
 	if (data->active == TRUE)
-		del_routes(data);
+		err = del_routes(data);
+	else
+		err = 0;
 
 	g_free(data->gateway);
 	g_free(data->vpn_ip);
 	g_free(data);
+
+	return err;
 }
 
 static void connection_delgateway(int index, const char *gateway)
@@ -229,6 +234,7 @@ static struct gateway_data *find_active_gateway(void)
 
 	for (list = gateway_list; list; list = list->next) {
 		struct gateway_data *data = list->data;
+
 		if (data->active == TRUE)
 			return data;
 	}
@@ -261,15 +267,21 @@ static int connection_probe(struct connman_element *element)
 
 	DBG("gateway %s", gateway);
 
+	/*
+	 * If gateway is NULL, it's a point to point link and the default
+	 * gateway is 0.0.0.0, meaning the interface.
+	 */
+	if (gateway == NULL) {
+		gateway = "0.0.0.0";
+		element->ipv4.gateway = g_strdup(gateway);
+	}
+
 	service = __connman_element_get_service(element);
 	__connman_service_indicate_state(service,
 					CONNMAN_SERVICE_STATE_READY);
 
 	connman_element_set_enabled(element, TRUE);
 
-	if (gateway == NULL)
-		return 0;
-
 	active_gateway = find_active_gateway();
 	new_gateway = add_gateway(element->index, gateway);
 
@@ -278,6 +290,8 @@ static int connection_probe(struct connman_element *element)
 		new_gateway->vpn_ip = g_strdup(vpn_ip);
 		/* make sure vpn gateway are at higher priority */
 		new_gateway->order = 10;
+		if (active_gateway)
+			new_gateway->vpn_phy_index = active_gateway->index;
 	} else
 		new_gateway->vpn = FALSE;
 
@@ -290,9 +304,6 @@ static int connection_probe(struct connman_element *element)
 		connman_inet_add_host_route_vpn(active_gateway->index,
 						active_gateway->gateway,
 						new_gateway->gateway);
-
-		connman_inet_set_gateway_address(new_gateway->index,
-							new_gateway->gateway);
 	}
 
 	if (new_gateway->order >= active_gateway->order) {
@@ -309,6 +320,7 @@ static void connection_remove(struct connman_element *element)
 	const char *gateway = NULL;
 	struct gateway_data *data = NULL;
 	gboolean set_default = FALSE;
+	int err;
 
 	DBG("element %p name %s", element, element->name);
 
@@ -332,16 +344,17 @@ static void connection_remove(struct connman_element *element)
 
 	set_default = data->vpn;
 
-	if (data->vpn == TRUE)
-		connman_inet_del_host_route(data->index, data->gateway);
+	if (data->vpn == TRUE && data->vpn_phy_index >= 0)
+		connman_inet_del_host_route(data->vpn_phy_index, data->gateway);
 
-	remove_gateway(data);
+	err = remove_gateway(data);
 
 	/* with vpn this will be called after the network was deleted,
 	 * we need to call set_default here because we will not recieve any
 	 * gateway delete notification.
+	 * We hit the same issue if remove_gateway() fails.
 	 */
-	if (set_default) {
+	if (set_default || err < 0) {
 		data = find_default_gateway();
 		if (data != NULL)
 			set_default_gateway(data);
