@@ -2,7 +2,7 @@
  *
  *  Connection Manager
  *
- *  Copyright (C) 2007-2009  Intel Corporation. All rights reserved.
+ *  Copyright (C) 2007-2010  Intel Corporation. All rights reserved.
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License version 2 as
@@ -35,14 +35,22 @@ DBusMessage *__connman_error_failed(DBusMessage *msg, int errnum)
 	const char *str = strerror(errnum);
 
 	switch (errnum) {
+	case ESRCH:
+		return __connman_error_not_registered(msg);
+	case ENXIO:
+		return __connman_error_not_found(msg);
 	case EACCES:
 		return __connman_error_permission_denied(msg);
+	case EEXIST:
+		return __connman_error_already_exists(msg);
 	case EINVAL:
 		return __connman_error_invalid_arguments(msg);
 	case ENOSYS:
 		return __connman_error_not_implemented(msg);
 	case ENOLINK:
 		return __connman_error_no_carrier(msg);
+	case ENOTUNIQ:
+		return __connman_error_not_unique(msg);
 	case EOPNOTSUPP:
 		return __connman_error_not_supported(msg);
 	case ECONNABORTED:
@@ -55,6 +63,8 @@ DBusMessage *__connman_error_failed(DBusMessage *msg, int errnum)
 		return __connman_error_operation_timeout(msg);
 	case EALREADY:
 		return __connman_error_in_progress(msg);
+	case ENOKEY:
+		return __connman_error_passphrase_required(msg);
 	}
 
 	return g_dbus_create_error(msg, CONNMAN_ERROR_INTERFACE
@@ -73,6 +83,24 @@ DBusMessage *__connman_error_permission_denied(DBusMessage *msg)
 				".PermissionDenied", "Permission denied");
 }
 
+DBusMessage *__connman_error_passphrase_required(DBusMessage *msg)
+{
+	return g_dbus_create_error(msg, CONNMAN_ERROR_INTERFACE
+				".PassphraseRequired", "Passphrase required");
+}
+
+DBusMessage *__connman_error_not_registered(DBusMessage *msg)
+{
+	return g_dbus_create_error(msg, CONNMAN_ERROR_INTERFACE
+					".NotRegistered", "Not registered");
+}
+
+DBusMessage *__connman_error_not_unique(DBusMessage *msg)
+{
+	return g_dbus_create_error(msg, CONNMAN_ERROR_INTERFACE
+					".NotUnique", "Not unique");
+}
+
 DBusMessage *__connman_error_not_supported(DBusMessage *msg)
 {
 	return g_dbus_create_error(msg, CONNMAN_ERROR_INTERFACE
@@ -85,6 +113,12 @@ DBusMessage *__connman_error_not_implemented(DBusMessage *msg)
 					".NotImplemented", "Not implemented");
 }
 
+DBusMessage *__connman_error_not_found(DBusMessage *msg)
+{
+	return g_dbus_create_error(msg, CONNMAN_ERROR_INTERFACE
+						".NotFound", "Not found");
+}
+
 DBusMessage *__connman_error_no_carrier(DBusMessage *msg)
 {
 	return g_dbus_create_error(msg, CONNMAN_ERROR_INTERFACE
@@ -97,6 +131,12 @@ DBusMessage *__connman_error_in_progress(DBusMessage *msg)
 						".InProgress", "In progress");
 }
 
+DBusMessage *__connman_error_already_exists(DBusMessage *msg)
+{
+	return g_dbus_create_error(msg, CONNMAN_ERROR_INTERFACE
+				".AlreadyExists", "Already exists");
+}
+
 DBusMessage *__connman_error_already_enabled(DBusMessage *msg)
 {
 	return g_dbus_create_error(msg, CONNMAN_ERROR_INTERFACE
