@@ -2,7 +2,7 @@
  *
  *  Connection Manager
  *
- *  Copyright (C) 2007-2009  Intel Corporation. All rights reserved.
+ *  Copyright (C) 2007-2010  Intel Corporation. All rights reserved.
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License version 2 as
@@ -25,6 +25,7 @@
 
 #include <net/if.h>
 #include <net/if_arp.h>
+#include <linux/if_link.h>
 
 #ifndef IFF_LOWER_UP
 #define IFF_LOWER_UP	0x10000
@@ -45,6 +46,7 @@ struct connman_ipconfig {
 
 	enum connman_ipconfig_method method;
 	struct connman_ipaddress *address;
+	struct connman_ipaddress *system;
 };
 
 struct connman_ipdevice {
@@ -52,6 +54,10 @@ struct connman_ipdevice {
 	char *ifname;
 	unsigned short type;
 	unsigned int flags;
+	char *address;
+	uint16_t mtu;
+	uint32_t tx_bytes;
+	uint32_t rx_bytes;
 
 	GSList *address_list;
 	char *gateway;
@@ -84,9 +90,64 @@ void connman_ipaddress_free(struct connman_ipaddress *ipaddress)
 	g_free(ipaddress->broadcast);
 	g_free(ipaddress->peer);
 	g_free(ipaddress->local);
+	g_free(ipaddress->gateway);
 	g_free(ipaddress);
 }
 
+static unsigned char netmask2prefixlen(const char *netmask)
+{
+	unsigned char bits = 0;
+	in_addr_t mask = inet_network(netmask);
+	in_addr_t host = ~mask;
+
+	/* a valid netmask must be 2^n - 1 */
+	if ((host & (host + 1)) != 0)
+		return -1;
+
+	for (; mask; mask <<= 1)
+		++bits;
+
+	return bits;
+}
+
+void connman_ipaddress_set(struct connman_ipaddress *ipaddress,
+		const char *address, const char *netmask, const char *gateway)
+{
+	if (ipaddress == NULL)
+		return;
+
+	if (netmask != NULL)
+		ipaddress->prefixlen = netmask2prefixlen(netmask);
+	else
+		ipaddress->prefixlen = 32;
+
+	g_free(ipaddress->local);
+	ipaddress->local = g_strdup(address);
+
+	g_free(ipaddress->gateway);
+	ipaddress->gateway = g_strdup(gateway);
+}
+
+void connman_ipaddress_clear(struct connman_ipaddress *ipaddress)
+{
+	if (ipaddress == NULL)
+		return;
+
+	ipaddress->prefixlen = 0;
+
+	g_free(ipaddress->local);
+	ipaddress->local = NULL;
+
+	g_free(ipaddress->peer);
+	ipaddress->peer = NULL;
+
+	g_free(ipaddress->broadcast);
+	ipaddress->broadcast = NULL;
+
+	g_free(ipaddress->gateway);
+	ipaddress->gateway = NULL;
+}
+
 void connman_ipaddress_copy(struct connman_ipaddress *ipaddress,
 					struct connman_ipaddress *source)
 {
@@ -103,6 +164,9 @@ void connman_ipaddress_copy(struct connman_ipaddress *ipaddress,
 
 	g_free(ipaddress->broadcast);
 	ipaddress->broadcast = g_strdup(source->broadcast);
+
+	g_free(ipaddress->gateway);
+	ipaddress->gateway = g_strdup(source->gateway);
 }
 
 static void free_address_list(struct connman_ipdevice *ipdevice)
@@ -179,6 +243,7 @@ static void free_ipdevice(gpointer data)
 	free_address_list(ipdevice);
 	g_free(ipdevice->gateway);
 
+	g_free(ipdevice->address);
 	g_free(ipdevice->ifname);
 	g_free(ipdevice);
 }
@@ -235,8 +300,9 @@ static void __connman_ipconfig_lower_up(struct connman_ipdevice *ipdevice)
 
 	switch (ipdevice->config->method) {
 	case CONNMAN_IPCONFIG_METHOD_UNKNOWN:
-	case CONNMAN_IPCONFIG_METHOD_IGNORE:
-	case CONNMAN_IPCONFIG_METHOD_STATIC:
+	case CONNMAN_IPCONFIG_METHOD_OFF:
+	case CONNMAN_IPCONFIG_METHOD_FIXED:
+	case CONNMAN_IPCONFIG_METHOD_MANUAL:
 		return;
 	case CONNMAN_IPCONFIG_METHOD_DHCP:
 		break;
@@ -284,8 +350,31 @@ static void __connman_ipconfig_lower_down(struct connman_ipdevice *ipdevice)
 	connman_inet_clear_address(ipdevice->index);
 }
 
+static void update_stats(struct connman_ipdevice *ipdevice,
+						struct rtnl_link_stats *stats)
+{
+	if (stats->rx_packets == 0 && stats->tx_packets == 0)
+		return;
+
+	connman_info("%s {RX} %u packets %u bytes", ipdevice->ifname,
+					stats->rx_packets, stats->rx_bytes);
+	connman_info("%s {TX} %u packets %u bytes", ipdevice->ifname,
+					stats->tx_packets, stats->tx_bytes);
+
+	if (ipdevice->config == NULL)
+		return;
+
+	ipdevice->rx_bytes = stats->rx_bytes;
+	ipdevice->tx_bytes = stats->tx_bytes;
+
+	__connman_counter_notify(ipdevice->ifname,
+				ipdevice->rx_bytes, ipdevice->tx_bytes);
+}
+
 void __connman_ipconfig_newlink(int index, unsigned short type,
-							unsigned int flags)
+				unsigned int flags, const char *address,
+							unsigned short mtu,
+						struct rtnl_link_stats *stats)
 {
 	struct connman_ipdevice *ipdevice;
 	GList *list;
@@ -295,6 +384,9 @@ void __connman_ipconfig_newlink(int index, unsigned short type,
 
 	DBG("index %d", index);
 
+	if (type == ARPHRD_LOOPBACK)
+		return;
+
 	ipdevice = g_hash_table_lookup(ipdevice_hash, GINT_TO_POINTER(index));
 	if (ipdevice != NULL)
 		goto update;
@@ -307,12 +399,18 @@ void __connman_ipconfig_newlink(int index, unsigned short type,
 	ipdevice->ifname = connman_inet_ifname(index);
 	ipdevice->type = type;
 
+	ipdevice->address = g_strdup(address);
+
 	g_hash_table_insert(ipdevice_hash, GINT_TO_POINTER(index), ipdevice);
 
 	connman_info("%s {create} index %d type %d <%s>", ipdevice->ifname,
 						index, type, type2str(type));
 
 update:
+	ipdevice->mtu = mtu;
+
+	update_stats(ipdevice, stats);
+
 	if (flags == ipdevice->flags)
 		return;
 
@@ -332,8 +430,6 @@ update:
 			lower_down = TRUE;
 	}
 
-	connman_inet_clear_address(index);
-
 	ipdevice->flags = flags;
 
 	str = g_string_new(NULL);
@@ -363,6 +459,9 @@ update:
 		if (index != ipconfig->index)
 			continue;
 
+		if (ipconfig->ops == NULL)
+			continue;
+
 		if (up == TRUE && ipconfig->ops->up)
 			ipconfig->ops->up(ipconfig);
 		if (lower_up == TRUE && ipconfig->ops->lower_up)
@@ -380,7 +479,7 @@ update:
 		__connman_ipconfig_lower_down(ipdevice);
 }
 
-void __connman_ipconfig_dellink(int index)
+void __connman_ipconfig_dellink(int index, struct rtnl_link_stats *stats)
 {
 	struct connman_ipdevice *ipdevice;
 	GList *list;
@@ -391,6 +490,8 @@ void __connman_ipconfig_dellink(int index)
 	if (ipdevice == NULL)
 		return;
 
+	update_stats(ipdevice, stats);
+
 	for (list = g_list_first(ipconfig_list); list;
 						list = g_list_next(list)) {
 		struct connman_ipconfig *ipconfig = list->data;
@@ -400,6 +501,9 @@ void __connman_ipconfig_dellink(int index)
 
 		ipconfig->index = -1;
 
+		if (ipconfig->ops == NULL)
+			continue;
+
 		if (ipconfig->ops->lower_down)
 			ipconfig->ops->lower_down(ipconfig);
 		if (ipconfig->ops->down)
@@ -437,6 +541,9 @@ void __connman_ipconfig_newaddr(int index, const char *label,
 	connman_info("%s {add} address %s/%u label %s", ipdevice->ifname,
 						address, prefixlen, label);
 
+	if (ipdevice->config != NULL)
+		connman_ipaddress_copy(ipdevice->config->system, ipaddress);
+
 	if ((ipdevice->flags & (IFF_RUNNING | IFF_LOWER_UP)) != (IFF_RUNNING | IFF_LOWER_UP))
 		return;
 
@@ -450,6 +557,9 @@ void __connman_ipconfig_newaddr(int index, const char *label,
 		if (index != ipconfig->index)
 			continue;
 
+		if (ipconfig->ops == NULL)
+			continue;
+
 		if (ipconfig->ops->ip_bound)
 			ipconfig->ops->ip_bound(ipconfig);
 	}
@@ -493,6 +603,9 @@ void __connman_ipconfig_deladdr(int index, const char *label,
 		if (index != ipconfig->index)
 			continue;
 
+		if (ipconfig->ops == NULL)
+			continue;
+
 		if (ipconfig->ops->ip_release)
 			ipconfig->ops->ip_release(ipconfig);
 	}
@@ -510,8 +623,23 @@ void __connman_ipconfig_newroute(int index, unsigned char scope,
 		return;
 
 	if (scope == 0 && g_strcmp0(dst, "0.0.0.0") == 0) {
+		GSList *list;
+
 		g_free(ipdevice->gateway);
 		ipdevice->gateway = g_strdup(gateway);
+
+		if (ipdevice->config != NULL &&
+					ipdevice->config->system != NULL) {
+			g_free(ipdevice->config->system->gateway);
+			ipdevice->config->system->gateway = g_strdup(gateway);
+		}
+
+		for (list = ipdevice->address_list; list; list = list->next) {
+			struct connman_ipaddress *ipaddress = list->data;
+
+			g_free(ipaddress->gateway);
+			ipaddress->gateway = g_strdup(gateway);
+		}
 	}
 
 	connman_info("%s {add} route %s gw %s scope %u <%s>",
@@ -531,8 +659,23 @@ void __connman_ipconfig_delroute(int index, unsigned char scope,
 		return;
 
 	if (scope == 0 && g_strcmp0(dst, "0.0.0.0") == 0) {
+		GSList *list;
+
 		g_free(ipdevice->gateway);
 		ipdevice->gateway = NULL;
+
+		if (ipdevice->config != NULL &&
+					ipdevice->config->system != NULL) {
+			g_free(ipdevice->config->system->gateway);
+			ipdevice->config->system->gateway = NULL;
+		}
+
+		for (list = ipdevice->address_list; list; list = list->next) {
+			struct connman_ipaddress *ipaddress = list->data;
+
+			g_free(ipaddress->gateway);
+			ipaddress->gateway = NULL;
+		}
 	}
 
 	connman_info("%s {del} route %s gw %s scope %u <%s>",
@@ -591,6 +734,11 @@ const char *__connman_ipconfig_get_gateway(int index)
 	return ipdevice->gateway;
 }
 
+void __connman_ipconfig_set_index(struct connman_ipconfig *ipconfig, int index)
+{
+	ipconfig->index = index;
+}
+
 /**
  * connman_ipconfig_create:
  *
@@ -618,6 +766,8 @@ struct connman_ipconfig *connman_ipconfig_create(int index)
 		return NULL;
 	}
 
+	ipconfig->system = connman_ipaddress_alloc();
+
 	DBG("ipconfig %p", ipconfig);
 
 	return ipconfig;
@@ -671,6 +821,8 @@ struct connman_ipconfig *connman_ipconfig_ref(struct connman_ipconfig *ipconfig)
 void connman_ipconfig_unref(struct connman_ipconfig *ipconfig)
 {
 	if (g_atomic_int_dec_and_test(&ipconfig->refcount) == TRUE) {
+		__connman_ipconfig_disable(ipconfig);
+
 		connman_ipconfig_set_ops(ipconfig, NULL);
 
 		if (ipconfig->origin != NULL) {
@@ -678,6 +830,7 @@ void connman_ipconfig_unref(struct connman_ipconfig *ipconfig)
 			ipconfig->origin = NULL;
 		}
 
+		connman_ipaddress_free(ipconfig->system);
 		connman_ipaddress_free(ipconfig->address);
 		g_free(ipconfig);
 	}
@@ -714,6 +867,9 @@ void connman_ipconfig_set_data(struct connman_ipconfig *ipconfig, void *data)
  */
 int connman_ipconfig_get_index(struct connman_ipconfig *ipconfig)
 {
+	if (ipconfig == NULL)
+		return -1;
+
 	if (ipconfig->origin != NULL)
 		return ipconfig->origin->index;
 
@@ -730,6 +886,9 @@ const char *connman_ipconfig_get_ifname(struct connman_ipconfig *ipconfig)
 {
 	struct connman_ipdevice *ipdevice;
 
+	if (ipconfig == NULL)
+		return NULL;
+
 	if (ipconfig->index < 0)
 		return NULL;
 
@@ -751,13 +910,7 @@ const char *connman_ipconfig_get_ifname(struct connman_ipconfig *ipconfig)
 void connman_ipconfig_set_ops(struct connman_ipconfig *ipconfig,
 				const struct connman_ipconfig_ops *ops)
 {
-	if (ipconfig->ops != NULL)
-		ipconfig_list = g_list_remove(ipconfig_list, ipconfig);
-
 	ipconfig->ops = ops;
-
-	if (ops != NULL)
-		ipconfig_list = g_list_append(ipconfig_list, ipconfig);
 }
 
 /**
@@ -775,6 +928,14 @@ int connman_ipconfig_set_method(struct connman_ipconfig *ipconfig,
 	return 0;
 }
 
+enum connman_ipconfig_method __connman_ipconfig_get_method(struct connman_ipconfig *ipconfig)
+{
+	if (ipconfig == NULL)
+		return CONNMAN_IPCONFIG_METHOD_UNKNOWN;
+
+	return ipconfig->method;
+}
+
 /**
  * connman_ipconfig_bind:
  * @ipconfig: ipconfig structure
@@ -794,6 +955,65 @@ void connman_ipconfig_bind(struct connman_ipconfig *ipconfig,
 	connman_inet_set_address(origin->index, origin->address);
 }
 
+/* FIXME: The element soulution should be removed in the future */
+int __connman_ipconfig_set_gateway(struct connman_ipconfig *ipconfig,
+						struct connman_element *parent)
+{
+	struct connman_element *connection;
+
+	connection = connman_element_create(NULL);
+
+	connection->type  = CONNMAN_ELEMENT_TYPE_CONNECTION;
+	connection->index = ipconfig->index;
+	connection->ipv4.gateway = ipconfig->address->gateway;
+
+	if (connman_element_register(connection, parent) < 0)
+		connman_element_unref(connection);
+
+	return 0;
+}
+
+int __connman_ipconfig_set_address(struct connman_ipconfig *ipconfig)
+{
+	DBG("");
+
+	switch (ipconfig->method) {
+	case CONNMAN_IPCONFIG_METHOD_UNKNOWN:
+	case CONNMAN_IPCONFIG_METHOD_OFF:
+	case CONNMAN_IPCONFIG_METHOD_FIXED:
+	case CONNMAN_IPCONFIG_METHOD_DHCP:
+		break;
+	case CONNMAN_IPCONFIG_METHOD_MANUAL:
+		return connman_inet_set_address(ipconfig->index,
+						ipconfig->address);
+	}
+
+	return 0;
+}
+
+int __connman_ipconfig_clear_address(struct connman_ipconfig *ipconfig)
+{
+	DBG("");
+
+	if (ipconfig == NULL)
+		return 0;
+
+	DBG("method %d", ipconfig->method);
+
+	switch (ipconfig->method) {
+	case CONNMAN_IPCONFIG_METHOD_UNKNOWN:
+	case CONNMAN_IPCONFIG_METHOD_OFF:
+	case CONNMAN_IPCONFIG_METHOD_FIXED:
+	case CONNMAN_IPCONFIG_METHOD_DHCP:
+		break;
+	case CONNMAN_IPCONFIG_METHOD_MANUAL:
+		return connman_inet_clear_address(ipconfig->index);
+	}
+
+	return 0;
+
+}
+
 int __connman_ipconfig_enable(struct connman_ipconfig *ipconfig)
 {
 	struct connman_ipdevice *ipdevice;
@@ -808,11 +1028,21 @@ int __connman_ipconfig_enable(struct connman_ipconfig *ipconfig)
 	if (ipdevice == NULL)
 		return -ENXIO;
 
-	if (ipdevice->config != NULL)
+	if (ipdevice->config == ipconfig)
+		return -EALREADY;
+
+	if (ipdevice->config != NULL) {
+		ipconfig_list = g_list_remove(ipconfig_list, ipconfig);
+
+		connman_ipaddress_clear(ipdevice->config->system);
+
 		connman_ipconfig_unref(ipdevice->config);
+	}
 
 	ipdevice->config = connman_ipconfig_ref(ipconfig);
 
+	ipconfig_list = g_list_append(ipconfig_list, ipconfig);
+
 	return 0;
 }
 
@@ -833,6 +1063,10 @@ int __connman_ipconfig_disable(struct connman_ipconfig *ipconfig)
 	if (ipdevice->config == NULL || ipdevice->config != ipconfig)
 		return -EINVAL;
 
+	ipconfig_list = g_list_remove(ipconfig_list, ipconfig);
+
+	connman_ipaddress_clear(ipdevice->config->system);
+
 	connman_ipconfig_unref(ipdevice->config);
 	ipdevice->config = NULL;
 
@@ -844,10 +1078,12 @@ const char *__connman_ipconfig_method2string(enum connman_ipconfig_method method
 	switch (method) {
 	case CONNMAN_IPCONFIG_METHOD_UNKNOWN:
 		break;
-	case CONNMAN_IPCONFIG_METHOD_IGNORE:
-		return "ignore";
-	case CONNMAN_IPCONFIG_METHOD_STATIC:
-		return "static";
+	case CONNMAN_IPCONFIG_METHOD_OFF:
+		return "off";
+	case CONNMAN_IPCONFIG_METHOD_FIXED:
+		return "fixed";
+	case CONNMAN_IPCONFIG_METHOD_MANUAL:
+		return "manual";
 	case CONNMAN_IPCONFIG_METHOD_DHCP:
 		return "dhcp";
 	}
@@ -857,35 +1093,20 @@ const char *__connman_ipconfig_method2string(enum connman_ipconfig_method method
 
 enum connman_ipconfig_method __connman_ipconfig_string2method(const char *method)
 {
-	if (g_strcmp0(method, "ignore") == 0)
-		return CONNMAN_IPCONFIG_METHOD_IGNORE;
-	else if (g_strcmp0(method, "static") == 0)
-		return CONNMAN_IPCONFIG_METHOD_STATIC;
+	if (g_strcmp0(method, "off") == 0)
+		return CONNMAN_IPCONFIG_METHOD_OFF;
+	else if (g_strcmp0(method, "fixed") == 0)
+		return CONNMAN_IPCONFIG_METHOD_FIXED;
+	else if (g_strcmp0(method, "manual") == 0)
+		return CONNMAN_IPCONFIG_METHOD_MANUAL;
 	else if (g_strcmp0(method, "dhcp") == 0)
 		return CONNMAN_IPCONFIG_METHOD_DHCP;
 	else
 		return CONNMAN_IPCONFIG_METHOD_UNKNOWN;
 }
 
-static void append_variant(DBusMessageIter *iter, const char *prefix,
-					const char *key, int type, void *val)
-{
-	char *str;
-
-	if (prefix == NULL) {
-		connman_dbus_dict_append_variant(iter, key, type, val);
-		return;
-	}
-
-	str = g_strdup_printf("%s%s", prefix, key);
-	if (str != NULL)
-		connman_dbus_dict_append_variant(iter, str, type, val);
-
-	g_free(str);
-}
-
 void __connman_ipconfig_append_ipv4(struct connman_ipconfig *ipconfig,
-				DBusMessageIter *iter, const char *prefix)
+							DBusMessageIter *iter)
 {
 	const char *str;
 
@@ -893,44 +1114,283 @@ void __connman_ipconfig_append_ipv4(struct connman_ipconfig *ipconfig,
 	if (str == NULL)
 		return;
 
-	append_variant(iter, prefix, "Method", DBUS_TYPE_STRING, &str);
+	connman_dbus_dict_append_basic(iter, "Method", DBUS_TYPE_STRING, &str);
+
+	if (ipconfig->system == NULL)
+		return;
+
+	if (ipconfig->system->local != NULL) {
+		in_addr_t addr;
+		struct in_addr netmask;
+		char *mask;
+
+		connman_dbus_dict_append_basic(iter, "Address",
+				DBUS_TYPE_STRING, &ipconfig->system->local);
+
+		addr = 0xffffffff << (32 - ipconfig->system->prefixlen);
+		netmask.s_addr = htonl(addr);
+		mask = inet_ntoa(netmask);
+		connman_dbus_dict_append_basic(iter, "Netmask",
+						DBUS_TYPE_STRING, &mask);
+	}
+
+	if (ipconfig->system->gateway != NULL)
+		connman_dbus_dict_append_basic(iter, "Gateway",
+				DBUS_TYPE_STRING, &ipconfig->system->gateway);
 }
 
-int __connman_ipconfig_set_ipv4(struct connman_ipconfig *ipconfig,
-				const char *key, DBusMessageIter *value)
+void __connman_ipconfig_append_ipv4config(struct connman_ipconfig *ipconfig,
+							DBusMessageIter *iter)
 {
-	int type = dbus_message_iter_get_arg_type(value);
+	const char *str;
 
-	DBG("ipconfig %p key %s type %d", ipconfig, key, type);
+	str = __connman_ipconfig_method2string(ipconfig->method);
+	if (str == NULL)
+		return;
 
-	if (g_strcmp0(key, "Method") == 0) {
-		const char *method;
+	connman_dbus_dict_append_basic(iter, "Method", DBUS_TYPE_STRING, &str);
 
-		if (type != DBUS_TYPE_STRING)
-			return -EINVAL;
+	switch (ipconfig->method) {
+	case CONNMAN_IPCONFIG_METHOD_UNKNOWN:
+	case CONNMAN_IPCONFIG_METHOD_OFF:
+	case CONNMAN_IPCONFIG_METHOD_FIXED:
+	case CONNMAN_IPCONFIG_METHOD_DHCP:
+		return;
+	case CONNMAN_IPCONFIG_METHOD_MANUAL:
+		break;
+	}
 
-		dbus_message_iter_get_basic(value, &method);
+	if (ipconfig->address == NULL)
+		return;
 
-		ipconfig->method = __connman_ipconfig_string2method(method);
-	} else
+	if (ipconfig->address->local != NULL) {
+		in_addr_t addr;
+		struct in_addr netmask;
+		char *mask;
+
+		connman_dbus_dict_append_basic(iter, "Address",
+				DBUS_TYPE_STRING, &ipconfig->address->local);
+
+		addr = 0xffffffff << (32 - ipconfig->address->prefixlen);
+		netmask.s_addr = htonl(addr);
+		mask = inet_ntoa(netmask);
+		connman_dbus_dict_append_basic(iter, "Netmask",
+						DBUS_TYPE_STRING, &mask);
+	}
+
+	if (ipconfig->address->gateway != NULL)
+		connman_dbus_dict_append_basic(iter, "Gateway",
+				DBUS_TYPE_STRING, &ipconfig->address->gateway);
+}
+
+int __connman_ipconfig_set_ipv4config(struct connman_ipconfig *ipconfig,
+							DBusMessageIter *array)
+{
+	enum connman_ipconfig_method method = CONNMAN_IPCONFIG_METHOD_UNKNOWN;
+	const char *address = NULL, *netmask = NULL, *gateway = NULL;
+	DBusMessageIter dict;
+
+	DBG("ipconfig %p", ipconfig);
+
+	if (dbus_message_iter_get_arg_type(array) != DBUS_TYPE_ARRAY)
 		return -EINVAL;
 
+	dbus_message_iter_recurse(array, &dict);
+
+	while (dbus_message_iter_get_arg_type(&dict) == DBUS_TYPE_DICT_ENTRY) {
+		DBusMessageIter entry;
+		const char *key;
+		int type;
+
+		dbus_message_iter_recurse(&dict, &entry);
+
+		if (dbus_message_iter_get_arg_type(&entry) != DBUS_TYPE_STRING)
+			return -EINVAL;
+
+		dbus_message_iter_get_basic(&entry, &key);
+		dbus_message_iter_next(&entry);
+
+		type = dbus_message_iter_get_arg_type(&entry);
+
+		if (g_str_equal(key, "Method") == TRUE) {
+			const char *str;
+
+			if (type != DBUS_TYPE_STRING)
+				return -EINVAL;
+
+			dbus_message_iter_get_basic(&entry, &str);
+			method = __connman_ipconfig_string2method(str);
+		} else if (g_str_equal(key, "Address") == TRUE) {
+			if (type != DBUS_TYPE_STRING)
+				return -EINVAL;
+
+			dbus_message_iter_get_basic(&entry, &address);
+		} else if (g_str_equal(key, "Netmask") == TRUE) {
+			if (type != DBUS_TYPE_STRING)
+				return -EINVAL;
+
+			dbus_message_iter_get_basic(&entry, &netmask);
+		} else if (g_str_equal(key, "Gateway") == TRUE) {
+			if (type != DBUS_TYPE_STRING)
+				return -EINVAL;
+
+			dbus_message_iter_get_basic(&entry, &gateway);
+		}
+		dbus_message_iter_next(&dict);
+	}
+
+	DBG("method %d address %s netmask %s gateway %s",
+				method, address, netmask, gateway);
+
+	switch (method) {
+	case CONNMAN_IPCONFIG_METHOD_UNKNOWN:
+	case CONNMAN_IPCONFIG_METHOD_OFF:
+	case CONNMAN_IPCONFIG_METHOD_FIXED:
+		return -EINVAL;
+
+	case CONNMAN_IPCONFIG_METHOD_MANUAL:
+		if (address == NULL)
+			return -EINVAL;
+
+		ipconfig->method = method;
+		connman_ipaddress_set(ipconfig->address,
+				address, netmask, gateway);
+		break;
+
+	case CONNMAN_IPCONFIG_METHOD_DHCP:
+		if (ipconfig->method == method)
+			return 0;
+
+		ipconfig->method = method;
+		break;
+	}
+
 	return 0;
 }
 
+void __connman_ipconfig_append_proxy(struct connman_ipconfig *ipconfig,
+							DBusMessageIter *iter)
+{
+	const char *method = "direct";
+
+	connman_dbus_dict_append_basic(iter, "Method",
+						DBUS_TYPE_STRING, &method);
+}
+
+void __connman_ipconfig_append_ethernet(struct connman_ipconfig *ipconfig,
+							DBusMessageIter *iter)
+{
+	struct connman_ipdevice *ipdevice;
+	const char *method = "auto";
+
+	connman_dbus_dict_append_basic(iter, "Method",
+						DBUS_TYPE_STRING, &method);
+
+	ipdevice = g_hash_table_lookup(ipdevice_hash,
+					GINT_TO_POINTER(ipconfig->index));
+	if (ipdevice == NULL)
+		return;
+
+	if (ipdevice->ifname != NULL)
+		connman_dbus_dict_append_basic(iter, "Interface",
+					DBUS_TYPE_STRING, &ipdevice->ifname);
+
+	if (ipdevice->address != NULL)
+		connman_dbus_dict_append_basic(iter, "Address",
+					DBUS_TYPE_STRING, &ipdevice->address);
+
+	if (ipdevice->mtu > 0)
+		connman_dbus_dict_append_basic(iter, "MTU",
+					DBUS_TYPE_UINT16, &ipdevice->mtu);
+}
+
 int __connman_ipconfig_load(struct connman_ipconfig *ipconfig,
 		GKeyFile *keyfile, const char *identifier, const char *prefix)
 {
+	const char *method;
+	char *key;
+
 	DBG("ipconfig %p identifier %s", ipconfig, identifier);
 
+	key = g_strdup_printf("%smethod", prefix);
+	method = g_key_file_get_string(keyfile, identifier, key, NULL);
+	if (method == NULL)
+		ipconfig->method = CONNMAN_IPCONFIG_METHOD_DHCP;
+	else
+		ipconfig->method = __connman_ipconfig_string2method(method);
+	g_free(key);
+
+	key = g_strdup_printf("%snetmask_prefixlen", prefix);
+	ipconfig->address->prefixlen = g_key_file_get_integer(
+				keyfile, identifier, key, NULL);
+	g_free(key);
+
+	key = g_strdup_printf("%slocal_address", prefix);
+	ipconfig->address->local = g_key_file_get_string(
+			keyfile, identifier, key, NULL);
+	g_free(key);
+
+	key = g_strdup_printf("%speer_address", prefix);
+	ipconfig->address->peer = g_key_file_get_string(
+				keyfile, identifier, key, NULL);
+	g_free(key);
+
+	key = g_strdup_printf("%sbroadcast_address", prefix);
+	ipconfig->address->broadcast = g_key_file_get_string(
+				keyfile, identifier, key, NULL);
+	g_free(key);
+
+	key = g_strdup_printf("%sgateway", prefix);
+	ipconfig->address->gateway = g_key_file_get_string(
+				keyfile, identifier, key, NULL);
+	g_free(key);
+
 	return 0;
 }
 
 int __connman_ipconfig_save(struct connman_ipconfig *ipconfig,
 		GKeyFile *keyfile, const char *identifier, const char *prefix)
 {
+	const char *method;
+	char *key;
+
 	DBG("ipconfig %p identifier %s", ipconfig, identifier);
 
+	method = __connman_ipconfig_method2string(ipconfig->method);
+
+	key = g_strdup_printf("%smethod", prefix);
+	g_key_file_set_string(keyfile, identifier, key, method);
+	g_free(key);
+
+	key = g_strdup_printf("%snetmask_prefixlen", prefix);
+	g_key_file_set_integer(keyfile, identifier,
+			key, ipconfig->address->prefixlen);
+	g_free(key);
+
+	key = g_strdup_printf("%slocal_address", prefix);
+	if (ipconfig->address->local != NULL)
+		g_key_file_set_string(keyfile, identifier,
+				key, ipconfig->address->local);
+	g_free(key);
+
+	key = g_strdup_printf("%speer_address", prefix);
+	if (ipconfig->address->peer != NULL)
+		g_key_file_set_string(keyfile, identifier,
+				key, ipconfig->address->peer);
+	g_free(key);
+
+	key = g_strdup_printf("%sbroadcast_address", prefix);
+	if (ipconfig->address->broadcast != NULL)
+		g_key_file_set_string(keyfile, identifier,
+			key, ipconfig->address->broadcast);
+	g_free(key);
+
+	key = g_strdup_printf("%sgateway", prefix);
+	if (ipconfig->address->gateway != NULL)
+		g_key_file_set_string(keyfile, identifier,
+			key, ipconfig->address->gateway);
+	g_free(key);
+
 	return 0;
 }
 
